@@ -140,11 +140,7 @@ extern  libGAP_Int             (*libGAP_IsbListFuncs[libGAP_LAST_REAL_TNUM+1]) (
 
 extern  libGAP_Int             (*libGAP_IsbvListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Int pos );
 
-#define libGAP_ISBB_LIST(list,pos) \
-                        ((*libGAP_IsbbListFuncs[libGAP_TNUM_OBJ(list)])(list,pos))
-
-extern  libGAP_Int             (*libGAP_IsbbListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Obj pos );
-
+extern libGAP_Int libGAP_ISBB_LIST( libGAP_Obj list, libGAP_Obj pos );
 
 
 /****************************************************************************
@@ -221,38 +217,6 @@ extern libGAP_Obj (*libGAP_ElmListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj l
 /****************************************************************************
 **
 *F  ELM_LIST( <list>, <pos> ) . . . . . . . . . select an element from a list
-**
-**  'ELM_LIST' returns the element at the position  <pos> in the list <list>.
-**  An  error is signalled if  <list> is not a list,  if <pos> is larger than
-**  the length of <list>, or if <list>  has no assigned  object at <pos>.  It
-**  is the responsibility  of the caller to  ensure that <pos>  is a positive
-**  integer.
-**
-**  Note that 'ELM_LIST', 'ELMV_LIST', and  'ELMW_LIST' are macros, so do not
-**  call them with arguments that have side effects.
-**
-**  The difference between ELM_LIST and ELMB_LIST is that ELMB_LIST accepts
-**  an object as the second argument (which should be a positive large integer.
-**  It is intended as an interface for access to elements of large external
-**  lists, on the rare occasions when the kernel needs to do this.
-*/
-#define libGAP_ELM_LIST(list,pos)      ((*libGAP_ElmListFuncs[libGAP_TNUM_OBJ(list)])(list,pos))
-
-/****************************************************************************
-**
-*V  ElmbListFuncs[ <type> ]  . . . . . . . . . .  table of selection functions
-**
-**  A package implementing a  list  type <type> may  provide a  function for
-**  'ELMB_LIST' and install it  in 'ElmbListFuncs[<type>]'.  This function must
-**  signal an error if <pos> is larger than the length of <list> or if <list>
-**  has no assigned object at <pos>.
-
-extern Obj (*ElmbListFuncs[LAST_REAL_TNUM+1]) ( Obj list, Obj pos );
-
-*/
-
-/****************************************************************************
-**
 *F  ELMB_LIST( <list>, <pos> ) . . . . . . . . . select an element from a list
 **
 **  'ELM_LIST' returns the element at the position  <pos> in the list <list>.
@@ -265,14 +229,26 @@ extern Obj (*ElmbListFuncs[LAST_REAL_TNUM+1]) ( Obj list, Obj pos );
 **  call them with arguments that have side effects.
 **
 **  The difference between ELM_LIST and ELMB_LIST is that ELMB_LIST accepts
-**  an object as the second argument (which should be a positive large integer.
+**  an object as the second argument
 **  It is intended as an interface for access to elements of large external
 **  lists, on the rare occasions when the kernel needs to do this.
 */
-extern libGAP_Obj libGAP_ELMB_LIST( libGAP_Obj list, libGAP_Obj pos);
+#define libGAP_ELM_LIST(list,pos)      ((*libGAP_ElmListFuncs[libGAP_TNUM_OBJ(list)])(list,pos))
 
+extern libGAP_Obj libGAP_ELMB_LIST( libGAP_Obj list, libGAP_Obj pos );
 
+/****************************************************************************
+**
+*F  ELM2_LIST( <list>, <pos1>, <pos2> ) . . . . select an element from a list
+*/
 
+extern libGAP_Obj libGAP_Elm2List(libGAP_Obj list, libGAP_Obj pos1, libGAP_Obj pos2);
+
+#define libGAP_ELM2_LIST(list, pos1, pos2) libGAP_Elm2List(list, pos1, pos2)
+
+extern void libGAP_Ass2List(libGAP_Obj list, libGAP_Obj pos1, libGAP_Obj pos2, libGAP_Obj obj);
+
+#define libGAP_ASS2_LIST(list, pos1, pos2, obj) libGAP_Ass2List(list, pos1, pos2, obj)
 
 
 /****************************************************************************
@@ -415,12 +391,10 @@ extern void libGAP_ElmsListLevelCheck (
 
 extern void             (*libGAP_UnbListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Int pos );
 
-#define libGAP_UNBB_LIST(list,pos) \
-                        ((*libGAP_UnbbListFuncs[libGAP_TNUM_OBJ(list)])(list,pos))
+extern void libGAP_UNBB_LIST( libGAP_Obj list, libGAP_Obj pos );
 
-extern void             (*libGAP_UnbbListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Obj pos );
+extern void libGAP_UnbListDefault( libGAP_Obj list, libGAP_Int  pos );
 
-extern void  libGAP_UnbListDefault ( libGAP_Obj list, libGAP_Int  pos );
 
 /****************************************************************************
 **
@@ -446,29 +420,7 @@ extern void  libGAP_UnbListDefault ( libGAP_Obj list, libGAP_Int  pos );
 
 extern  void            (*libGAP_AssListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Int pos, libGAP_Obj obj );
 
-/****************************************************************************
-**
-*F  ASSB_LIST(<list>,<pos>,<obj>)  . . . . . . . . assign an element to a list
-*V  AssbListFuncs[<type>]  . . . . . . . . . . . table of assignment functions
-**
-**  'ASSB_LIST' assigns the object <obj> to the list <list> at position <pos>.
-**  Note that  the assignment may change  the length or the representation of
-**  <list>.  An error   is signalled if  <list>  is not a  list.    It is the
-**  responsibility of the caller to ensure that <pos>  is a positive integer,
-**  and that <obj> is not 0.
-**
-**  Note that 'ASSB_LIST' is a macro,  so do not  call it with arguments  that
-**  have side effects.
-**
-**  A package  implementing a list type  <type> must provide  such a function
-**  and   install it in  'AssbListFuncs[<type>]'.   This  function must extend
-**  <list> if <pos> is larger than the length of  <list> and must also change
-**  the representation of <list> to that of a plain list if necessary.
-*/
-#define libGAP_ASSB_LIST(list,pos,obj) \
-                        ((*libGAP_AssbListFuncs[libGAP_TNUM_OBJ(list)])(list,pos,obj))
-
-extern  void            (*libGAP_AssbListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libGAP_Obj list, libGAP_Obj pos, libGAP_Obj obj );
+extern void libGAP_ASSB_LIST( libGAP_Obj list, libGAP_Obj pos, libGAP_Obj obj );
 
 
 /****************************************************************************
@@ -821,7 +773,7 @@ extern  void            (*libGAP_PlainListFuncs[libGAP_LAST_REAL_TNUM+1]) ( libG
 
 /****************************************************************************
 **
-*F  TYPES_LIST_FAM(<fam>) . . . . . . .  list of kinds of lists over a family
+*F  TYPES_LIST_FAM(<fam>) . . . . . . .  list of types of lists over a family
 */
 extern  libGAP_Obj             libGAP_TYPES_LIST_FAM (
             libGAP_Obj                 fam );
@@ -852,7 +804,7 @@ extern libGAP_UInt libGAP_SetFiltListTNums [ libGAP_LAST_REAL_TNUM ] [ libGAP_LA
     libGAP_UInt     new; \
     new = libGAP_SetFiltListTNums[libGAP_TNUM_OBJ(list)][fn]; \
     if ( new != (libGAP_UInt)-1 ) \
-      libGAP_RetypeBag( list, new ); \
+      libGAP_RetypeBagIfWritable( list, new ); \
      else { \
       libGAP_Pr( "#E  SET_FILT_LIST[%s][%d] in ", (libGAP_Int)libGAP_TNAM_OBJ(list), fn ); \
       libGAP_Pr( "%s line %d\n", (libGAP_Int)__FILE__, (libGAP_Int)__LINE__); \
